#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use HTML::Template;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Config qw(%config expand_filename);
use PureMessage::Manager::Response::Policy::Repository;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Widget::Grid;
use PureMessage::Manager::Lang qw(text);
use PureMessage::Policy::Repository;
use PureMessage::Policy::Repository::File;
use PureMessage::Version qw($PREFIX);
use Data::Dumper;
my %vars;

my @files; # populated by generate_list_data
my @data;

my $WIDGET = new PureMessage::Manager::Widget;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Policy::Repository',
                                'stderr_to_stdout' =>1
                              );

my $SCRIPTNAME ="repository.cgi";

my $script = $in{f};
$r->{debug} = 0; 

my $script_anchor = $script || ''; 
$script_anchor =~ s/\.siv$// if $script;

$r->add_help(text => "Help", tab => "pol", page => "mgr_pol_repos" );

$WIDGET->read_input($REQUEST);
my $input_data = $WIDGET->result_data();

if ($in{use} || $input_data->{use}) {
  
  my $file = PureMessage::Policy::Repository::File->Load($script);
  
  if (!$file) {
    $r->raise_error(text('unable_to_open_sieve_file'),$script);
  } else {
      
    eval { 
      $file->to_clipboard();
      $r->addInfoBox(msg => text('file_used_as_clipboard',$file->name));
    };
    
    if($@) {
      $r->raise_error(text('unable_to_copy_to_clipboard',$@));
    } else {
      $script = undef;
    }
  }
  
  show_list();

} elsif($in{new} || $input_data->{new}) {
  
  eval {  
    my $file = PureMessage::Policy::Repository::File->Create();
    $file->from_clipboard();
    $script = $file->file();
  };
  if ($@) {
    $r->raise_error(text('unable_to_copy_from_clipboard',$@));
    $script = 'clipboard';
  }
  
  show_clip();
  
} elsif($in{update} || $input_data->{update}) {
  
  eval {
    my $file = PureMessage::Policy::Repository::File->Load($script);
  
    if (validData(\%in)) {
      
      $file->description($in{description});
    
      if($in{filename} ne $file->name()) {
        my $fname = $in{filename};
        $fname =~ s/ /_/g;
        $fname .= ".siv";
        $file->rename($fname);
        $script = $fname;
      }
    
      show_list();
    
    } else {
        
      show_clip(1);
    
    }
  
  };
  if($@) {
    $r->raise_error(text('unable_to_update_clip',$script,$@));
    show_clip(1);
  }
  
} elsif($in{cancel}|| $input_data->{cancel}) {
    
  show_list();

} elsif($in{aspolicy} || $input_data->{aspolicy}) {
 
  if (-e PureMessage::Policy::Repository::edit_location()) {
      
    $r->raise_error(text('uncommited_changes_must_be_reverted'));
    show_clip();
    
  } else {
    
    eval {
      my $file = PureMessage::Policy::Repository::->Edit(create => 1);
      $file->from_file($script);
      $r->addInfoBox(msg => text('file_used_as_policy_script',$file->name));
    };
    if($@) {
        
      $r->raise_error(text('unable_to_set_to_policy',$script));
      show_clip();
      
    } else {
        
      show_list();

    }
  
  }
    
} elsif($in{yesdelete} || $input_data->{yesdelete}) {
  
  # they confirmed that they would like to delete those items
  my @items = split(',',$in{delete_items});
  generate_list_data();
  
  foreach my $index (@items) {
    
    eval { $files[$index]->delete(); };
    if($@) {
      $r->raise_error('unable_to_delete_script',$@);
    }
    
  }
  
  generate_list_data();
  show_list();

} elsif($script) {
    
  show_clip();
  
} else {
    
  show_list();
  
}


sub show_clip{
    my $reload = shift;

    my (%form_data,$file,@fields,$options,@buttons);
    @fields = qw( Name Description Type Content);
    
    if ($script eq 'clipboard') {
        
      $file = PureMessage::Policy::Repository->Clipboard();
    
    } else {
        
      $file = PureMessage::Policy::Repository::File->Load($script);
    
    }

    unless($file) {
      $r->raise_error('unable_to_read_script',$script);
      $r->send();
      exit;
    }
    
    my %info = $file->info(); # will have {script} if full script
    
    my %TYPE = ( script  => "<img src='/img/policy_script.gif'>&nbsp;"
                          . text('complete_policy_file'),
                 snippet => "<img src='/img/paste.gif'>&nbsp;"
                          . text('snippet_file'),
               );
    
    my %data = ( description => $reload ? $in{description} 
                                        : $file->description(),
                        name => $reload ? $in{filename} 
                                        : $file->name(),
               );

    if($script eq 'clipboard') {
        
      $form_data{'Description'} = $data{description};
      $form_data{'Name'} = 'clipboard';
      push(@buttons,{ type=>'image',name=>'new',icon=>'new',text=>'Save to Repository' });
    
    } else {
      
      my $writable = -w $file->absfile();

      if ($writable) {
        $form_data{'Description'} = '<input type=text class=stretch name=description value="' . $data{description} . '">';
        $form_data{'Name'} = '<input type=text class=stretch name=filename value="' . $data{name} . '">';
      } else {
        $form_data{'Description'} = $data{description};
        $form_data{'Name'} = $data{name};
      }
      if ($info{script}) {
        push(@buttons,{ type=>'image',name=>'aspolicy',icon=>'ok',text=>'Use as Policy' });
      } else {
        push(@buttons,{ type=>'image',name=>'use',icon=>'ok',text=>'Copy To Clipboard' });
      }
      
      push(@buttons,{ type=>'image',name=>'update',icon=>'ok',text=>'Update' }) if $writable;
    
    }
    
    push(@buttons, { type=>'image',name=>'cancel',icon=>'cancel',text=>'Cancel' });
    
    $form_data{'Content'} = "<div style='background:white'>" 
                          . $r->format_source($file->absfile())
                          . "</div>";
                          
    $form_data{'Type'} = $info{script} ? $TYPE{script} : $TYPE{snippet};
    
    my $tb = $WIDGET->widget('CLASS' => 'PureMessage::Manager::Widget::ToolBar',
                              name=>'tb',
                              state => { f=> $script },
                              buttons => [\@buttons],
                             );

    my $form = $WIDGET->widget(CLASS => 'PureMessage::Manager::Widget::Form',
                               name => "form_data",
                               data => \%form_data,
                               columns => 1,
                               fields =>\@fields,
                               title => text('repository_form_title'),
                               action => $SCRIPTNAME,
                               width=>"98%",
                               keys => sub { return ucfirst(shift)},
                               toolbar => $tb,
                              );              
    
    $r->add_part($form,'form');
    $r->title(text('examine_policy_script'));
    $r->send();

}

sub generate_list_data {
  
  @data = ();
  
  eval {
    @files = PureMessage::Policy::Repository->Available();
    foreach my $file (@files) {
      my %info = $file->info(); # will have {script} if full script
      my ($name,$fn) = ($file->name(),$file->file());
      my $icon = $info{script} ? 'policy_script.gif' : 'paste.gif';
      $icon = "<img border=0 src='/img/$icon'>";
      my $link;
      $link .= "<a href='$SCRIPTNAME?f=$fn'>$_</a>&nbsp;" foreach ($icon,$name);
      my $desc = $file->description();
      push(@data, [$link,$desc ]);
    }
  };
  if($@) {
    $r->raise_error(text('error_loading_available_scripts',$@));
    $r->send();
    exit;
  }
}

sub show_list {
     
     my (@buttons,$special);
     
     generate_list_data();
     
     if (my $clipboard = PureMessage::Policy::Repository->Clipboard()) {
       $special = ["<a href='$SCRIPTNAME?f=clipboard'><img border=0 src='/img/paste.gif'></a>&nbsp;<a href='$SCRIPTNAME?f=clipboard'>[clipboard]</a>",$clipboard->description()];
       
     }
    
     push(@buttons, { type=>'image',name=>'delete',icon=>'delete',text=>'Delete' });
    
     my $tb = $WIDGET->widget('CLASS' => 'PureMessage::Manager::Widget::ToolBar',
                              name=>'tb',
                              state => {},
                              buttons => [\@buttons],
                             );


     my $grid = $WIDGET->widget(CLASS => 'PureMessage::Manager::Widget::Grid',
                                name => 'scripts',
                                data => \@data,
                                page => 0,
                                selectable=>1,
                                paging => 1,
                                page_size=>15,
                                title=>text('repository_grid_title'),
                                columns=>2,
                                width=> "98%",
                                toolbar => $tb,
                                special => $special,
                                action=>$SCRIPTNAME,
                                padding=>'<BR> <BR>',
                                ddata => [

                                           {
                                              width=>"20%",
                                              description=>'Name',
                                              extra=>'class=stretch nowrap',
                                            },
                                            {
                                              width=>"80%",
                                              description=>'Description',
                                              extra=>'nowrap',
                                            },

                                         ],
                              );


  $r->title(text('repository_list_title'));

  # see if there is input the grid widget should deal with
  if ($grid->read_input($REQUEST)) {
  
    my $data = $grid->result_data();
    
    if ($data->{delete}) {
      
      if ( exists($data->{selected_data_rows}) and defined($data->{selected_data_rows}) ) {
        
        my @selected;
        
        push(@selected,$data->{selected_data_rows})
          unless ref($data->{selected_data_rows}) eq 'ARRAY';
        
        @selected = @{$data->{selected_data_rows}} 
          if ref($data->{selected_data_rows}) eq 'ARRAY';
        
        my $delete_msg = text('please_confirm_delete');
        my $delete_items;
        foreach my $d (@selected) {
            $delete_items .= "<LI>" . $files[$d]->name() . "\n";
        }
        
        $delete_items ="<UL>$delete_items</UL>";
        my $delete_dlg_msg = <<EOF;
<table width="100%">
<tr>
  <td colspan=2>
$delete_msg
  </td>
<tr>
  <td width="30%">
&nbsp;
  </td>
  <td>
$delete_items
  </td>
</tr>
</table>
EOF

        $r->addDialog(dialog_title => text('confirm_delete_dialog_title'),
                   msg=>$delete_dlg_msg,
                   name=>'delete',
                   style => 'dlg_query',
                   buttons => [[ { type=>'image',name=>'yesdelete',icon=>'yes',text=>'Yes Delete' },
                                 { type=>'image',name=>'no',icon=>'no',text=>'No Cancel' }
                              ]],
                   state => { delete_items => join(',',@selected) },
                  );
        $r->send();
        return;
        
      } # if selected_data_rows has data
    } # data{delete}
  } # $grid->read_input

  $r->add_part($grid,'grid');
  $r->send();

}


sub validData {
  my $in = shift;
  
  foreach my $fixup ( qw( filename description) ) {
    # validator won't check keys that don't exists in input
    $in->{$fixup} = '' unless $in->{$fixup};  
  }
  
  $REQUEST->typedecl("filename", "#str #nomarkup #value", "Name");
  $REQUEST->typedecl("description", "#str #value #nomarkup", "Description");
  $REQUEST->typedecl("name", "#nomarkup #value", "Name");
  
  my @v_errors = $REQUEST->validate($in);
  
  if (@v_errors) {
      
      $r->raise_error(join("<br>", @v_errors));
      return;
      
  } else {
      return 1;
  }
  
}








