#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Config qw(%config);
use PureMessage::Config qw(%config expand_config_filename expand_filename);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Policy;
use PureMessage::Policy::TestStore;
use File::Basename;
use Net::Domain qw(hostfqdn);
use HTTP::Date;

my %vars;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Policy',
                                'stderr_to_stdout' =>1
                              );

$r->add_help(text => "Help", tab => "pol", page => "mgr_pol_test_current" );

$r->title(text('test_index_title'));

my @result;

my $file = "";

if ( $REQUEST->query('mbox') ) {
    $file = expand_filename("etc/data/samples/".$REQUEST->query('mbox'));
    unless ( -e $file ) {
        $r->raise_error(text('no_such_file', $file));
        $file = "";
    }
    $file = join(" ", <$file/*>);
}
elsif ( $REQUEST->query('mbox_data') ) {
    $file = "/tmp/mbox.$$.".time();
    if ( open (my $m, ">$file") ) {
        print $m $REQUEST->query('mbox_data');
    }
    else {
        $r->raise_error(text('cant_create', $file));
        $file = "";
    }
}

my $ip = $REQUEST->query('ip') || $REQUEST->query('relay');
my $from = $REQUEST->query('from');
my $to = $REQUEST->query('to');

if ( $file ) {
    my @buttons = ({type=>'text', href=>'/Policy/test.cgi', text=>'New Test'});
    my $tb = $r->ToolBar( buttons => [ \@buttons, ] );
    my ($details, $data);
    eval {
        ($details, $data) = run_test();
    };
    if ($@) {
        $r->raise_error($@);
    }
    else {
        $r->addGrid(
                    name => 'results',
                    data => sub {my $s = shift; return defined $s ? $data : scalar @$data},
                    title=>text('test_res_subtitle'),
                    columns=>3,
                    toolbar => $tb,
                    width=>"100%",
                    ddata => [
                                { 
                                  width=>100,
                                  description=>text('test_source'),
                                  extra=>'nowrap',
                                }, 
                                { 
                                  width=>200,
                                  description=>text('test_result'),
                                }, 
                                { 
                                  width=>200,
                                  description=>text('test_action'),
                                },
                             ],
                   );
    }
    $r->print($details);
}
else {
    $r->print_notification(text('about_test', 'current mail policy script'));
    my @buttons;
    push(@buttons, {type=>'image', icon=>"continue", name=>'test', text=>'Test'});
    my $tb = $r->ToolBar( buttons => [ \@buttons, ] );
    my ( $fields, $data ) = test_form();
    $r->addForm(
                name => "test",
                data => $data,
                fields => $fields,
                columns => 2,
                title => text('test_dlg_title'),
                toolbar => $tb,
                action => 'test.cgi',
                );
}

$r->send();

sub test_form {
    my $domain = hostfqdn() || 'localhost';
    my $localtime = localtime();
    my $rfc1123_time = time2str(time());
    my $example_msg_src = <<MSG;
To: PureMessage Test User <puremessage-test\@$domain>
From: $config{admin}
Subject: PureMessage Test
Date: $rfc1123_time

This message was generated to test PureMessage.
The current local time is: $localtime

Have a nice day,
$config{admin}
MSG
    
    my $fields =  [ 
                    text('relay'), 
                    text('env_from'),
                    text('or_ip'),
                    text('env_to'),
                    text('local_mbox'), undef,
                    text('paste_mbox'), undef, 
                  ];
    my $script = "<input type=hidden name=script value='".$REQUEST->query('script')."'>"
                if $REQUEST->query('script');
    my %ui = (
            text('relay')       => "<select class=stretch name=relay><option value='127.0.0.1'>Internal (localhost)<option value='123.123.123.123'>External</select>",
            text('or_ip')       => "<input name=ip class=stretch value=$ip>",
            text('env_from')    => "<input name=from class=stretch value=$from>",
            text('env_to')      => "<input name=to class=stretch value=$to>",
            text('local_mbox')  => samples_combo(),
            text('paste_mbox')  => "<textarea cols=40 rows=10 name=mbox_data style='width:100%'>$example_msg_src</textarea>",
            );

    return ($fields, \%ui);
    
}

sub samples_combo {
    my $msg = text('select_sample');
    my $vir = text('infected');
    my $spam = text('spam');
    my $normal = text('normal');
    
    return <<EOT;
<select class=stretch name=mbox>
<option value="">($msg)</option>
<option value="virus">$vir</option>
<option value="spam">Spam</option>
<option value="normal">Normal</option>
</select>
EOT
}

# from pmx-policy
sub _new_driver {
    require PureMessage::Policy::Engine::FileDriver;
    my $engine = shift;
    my %localopts = @_;
    return PureMessage::Policy::Engine::FileDriver->new(
        $engine, 
        %localopts,
        quarantine => PureMessage::Policy::TestStore->new(),
    );
}

sub _new_engine {
    require PureMessage::Policy::Engine;
    require PureMessage::Policy::Logger;
    my %localopts = @_;
    return PureMessage::Policy::Engine->new(
        script => $localopts{script} || expand_filename("etc/policy.siv"),
        plugins => [$localopts{config} || expand_filename("etc/data/policy/base.conf")],
        init => 1,
        logger=> logger(),
        %localopts,
    );
}

sub logger {
    my $class = shift;
    my %opts = @_;
    my $self = bless (\%opts, $class);
}
                
sub log {
    my @data = @_;
    splice(@data, 0, 2);
    my $line =  join(" ", @data);
    $line .= "\n" unless $line =~ /\n$/;
    push @result, $line;
}

sub run_test {
    my %opts;
    %opts = (script => expand_filename($REQUEST->query('script'))) 
            if $REQUEST->query('script');
    $opts{env_from} = $from if $from;
    if ($to) {
        my @to = split /\s*,\s*/, $to;
        $opts{env_to} = \@to;
    }

    if ($ip) {
        $opts{relay} = $ip;
        $opts{fur}   = $ip;
    }

    $opts{dry_run} = 1;
    my $engine = _new_engine(%opts);
    my $driver = _new_driver ($engine, %opts);
    my $rs = $driver->inject_file(split(" ", $file));
    my %stat;
    my $details = "<b>Details:</b> <br> <textarea cols=70 rows=10>" 
                . join("", @result) . "</textarea>";
    my @results;
    foreach my $res ( ($rs->results) ) {
        my $id = $res->source_id ? "[#".$res->source_id."]" : "";
        push @results, [
        substr(basename($res->source_file), 0, (18 - length $id)) . $id,
        '<a href="javascript:void(null)"'
        ." onClick=\"preview_window = window.open('/Quarantine/Manage/preview.cgi?row="
        . $res->id
        ."&message_set_id=-1&ms_class=PureMessage%3A%3APolicy%3A%3ATestStore', 'preview', 'width=430, height=560,scrollbars,resizable=yes')\""
        ." onMouseOver=\"old_status = window.status; window.status = 'Click to view message'; return true\""
        ." onMouseOut=\"window.status = old_status; return true\""
        ." >" . $res->id . "</a>",
        $res->action
        . (( $res->action eq 'quarantine') ? ": " . $res->reason : ""),
        ];
    
    }
    return $details, \@results;
}
