package PureMessage::Policy::Phrase::Config;

use strict;
use warnings;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = '%phrase_config';
our %phrase_config;

# Validation rules and default values.
my %phrase_conf = (
    'per_attachment_byte_limit'      => ['#size','64KB'],
    'per_message_time_limit'         => ['#num','256'],
);

refresh();

sub new {
    my $self = shift;
    my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Policy::Phrase::Config->validate($p);
    $p->parse_file(expand_filename("etc/phrase.conf"));
    $p->as_hash;
    $p;
}

sub refresh {
    %phrase_config = new(undef)->as_hash;
}

sub validate {
    my ($pkg, $p) = @_;

    # Add our validation rules to the parser.
    foreach my $key (keys %phrase_conf) {
        my ($type, $def) = @{$phrase_conf{$key}};
        $p->typedecl( $key, $type ) if (defined $type);
        $p->default(  $key, $def )  if (defined $def);
    }
}

1;

