package PureMessage::Policy::Repository::Config;
use strict;
use PureMessage::Config qw(expand_filename);
use PureMessage::Version qw($PREFIX);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = qw( %config %policy_repository_config);
our %config;
our $object;

our $policy_repository_config; # alias that doesn't clash with other %configs
*policy_repository_config =\%config;

our $CONFIG_FILE = 'etc/repository.conf';

sub refresh() {
    $object = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Policy::Repository::Config->validate($object);
    $object->parse_file(expand_filename($CONFIG_FILE));
    %config = $object->as_hash;
    $config{prefix} = $PREFIX;
    foreach (qw( log_dir)) {
      $config{$_} = $PureMessage::Config::config{$_};
    }
    $config{dir} = expand_filename($config{location}) unless $config{dir};
}

refresh();

=item edit()

 Will open this config as a writeable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
   my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
   PureMessage::Policy::Repository::Config->validate($p);
   $p->parse_file(expand_filename($CONFIG_FILE),writable => 1);
   return $p;
}

sub validate {
    my ($pkg, $p) = @_;
    $p->typedecl('location', '#str');
    $p->default('location','etc/data/policy/repository');
    $pkg->include_sub_validators($p);
}

1;
