package PureMessage::Policy::Template;
use strict;
use Carp qw(croak);
use PureMessage::Template qw(extract_templates expand_template);

sub new {
    my $class = shift;
    my $var_table = shift;
    croak "new() needs a hashref" unless ref ($var_table) eq 'HASH';
    return bless {global=>$var_table}, $class;
}

sub expand {
    my $self = shift;
    my $str_or_array = shift;
    my %opts = @_;
    
    if (!ref($str_or_array)) {
        return $self->_expand($str_or_array, %opts);
    }
    else {
        my @a = map $self->_expand($_, %opts), @{$str_or_array};
        return \@a;
    }
}

sub _expand {
    my $self = shift;
    my $str = shift;
    my %opts = @_;
    
    my %vars;
    my @list;
    
    # if we have a precompiled list of templates
    if ($opts{list}) {
        @list = @{$opts{list}};
    }
    else {
    # else create one
        @list = extract_templates($str);
    }
    
    # look up each template in our table
    foreach my $key (@list) {
        my $tmpl;
        
        $tmpl = $self->{global}{$key} if exists $self->{global}{$key};
        
        # local overrides global
        $tmpl = $self->{local}{$key} if exists $self->{local}{$key};

	# context templates
	my $t = $self->{context}{$key};
	if(ref($t) eq 'ARRAY') {
	    $tmpl = $t->[$#$t] if exists $t->[$#$t];
	}

        next unless defined $tmpl;
        
        # if it is a scalar, use it
        if (!ref($tmpl)) {
            $vars{lc $key} = $tmpl;
        }
        
        # if it is a coderef, run it
        elsif (ref ($tmpl) eq 'CODE') {
            $vars{lc $key} = $tmpl->(@{$self->{args} || []});
        }
        
        # else complain
        else {
            croak "bad template var found of type ".ref($tmpl);
        }
    }
    
    return expand_template($str, vars=>\%vars);
}

sub template_global {
    my $self = shift;
    my $tmpl = shift;
    
    return $self->{global} unless defined $tmpl;
    $self->{global} = $tmpl;   
}

sub template {
    my $self = shift;
    my $name = shift;
    my $value = shift;
    
    return $self->{local}{$name} if not defined $value;
    $self->{local}{$name} = $value;
}

sub template_context {
    my ($self, $name, $value) = @_;

    return $self->{context}{$name} unless defined $value;
    $self->{context}{$name} = $value;
}

sub template_reset {
    my $self = shift;
    delete $self->{local};
    delete $self->{args};
}

sub template_args {
    my $self = shift;
    my $args = shift;
    
    return $self->{args} unless defined $args;
    croak "template_args requires an arrayref" unless ref($args) eq "ARRAY";
    $self->{args} = $args;   
}

1;
