package PureMessage::Policy::VHash;
use strict;

sub new {
    my $class = shift;
    return bless {
	data => [],   # [($key, $view, $index, $value),... ]
	occur => {},  # $key/$count
	view => 0,
    }, $class;
}

sub get {
    my $self = shift;
    my $key = shift;
    my $index = shift || 0;
    my $lc_key = lc $key;
    
    my @values = ();
    my %found = ();
    my $data = $self->{data};
    for (my $i = 0; $i <= $#$data; $i+=4) {
        if (lc $data->[$i] eq $lc_key) {
            if (wantarray) {
                if (!exists $found{$data->[$i+2]}) {
                    push @values, $data->[$i+3];
                    $found{$data->[$i+2]} = 1;
                }
            }
            else {
                return $data->[$i+3] if $data->[$i+2] == $index;
            }
        } 
    }

    wantarray and return @values;
    return undef;
}

sub keys {
    my $self = shift;
    my %keys;
    my $data = $self->{data};
    my @ordered_keys;
    for (my $i = 0; $i <= $#$data; $i+=4) {
        my $key = $data->[$i];
        my $lc_key = lc $key;
        unless (exists $keys{$lc_key}) {
            push @ordered_keys, $key;
            $keys{$lc_key} = 1;
        }
    }
    return @ordered_keys;
}

sub exists {
    my $self = shift;
    my $key = shift;
    my $lc_key = lc $key;
    my $data = $self->{data};
    my %deleted;
    for (my $i = 0; $i <= $#$data; $i+=4) {
        if (lc $data->[$i] eq $lc_key) {
            # if this is a mod
            if ($data->[$i+1]) {
                # if its not a deletion
                if (defined $data->[$i+3]) {
                    # exists if was not deleted
                    return 1 unless $deleted{$data->[$i+2]};
                }
                else {
                    # track the deletion
                    $deleted{$data->[$i+2]} = 1;
                }
            }
            else {
                # exists if was not deleted
                return 1 unless $deleted{$data->[$i+2]};
            }
        }
    }
    return 0;
}

sub put {
    my $self = shift;
    my $key = shift;
    my $value = shift;
    my $index = shift || 0;
    my $lc_key = lc $key;
    my $data = $self->{data};
    my $view = $self->{view};
    my $found_key;
    
    for (my $i = 0; $i <= $#$data; $i+=4) {
        if (lc $data->[$i] eq $lc_key) {
            $found_key = $data->[$i];
            if ($data->[$i+2] == $index and $data->[$i+1] == $view)
            {
                $data->[$i] = $key;
                $data->[$i+3] = $value;
                delete $self->{dcache};
                return;
            }
        }
    }
    
    # not found - add new (to beginning of array so that mods are found first)
    unshift @{$data}, $key, $view, $index, $value;
    delete $self->{dcache};
}

sub append {
    my $self = shift;
    my $key = shift;
    my $value = shift;
    my $data = $self->{data};
    my $view = $self->{view};
    my $index = $self->{occur}->{$key} || 0;
    push @$data, $key, $view, $index, $value;
    $self->{occur}->{$key}++;    
    delete $self->{dcache};
}

sub view {
    my $self = shift;
    my $old = $self->{view};
    $self->{view} = shift if @_;
    $old;
}

sub dump {
    my $self = shift;
    $self->{dump_params} = 1 if @_;
    
    my $kvsep = shift || ": ";
    my $psep = shift || "\r\n";
    my $view = $self->{view};
    my $data = $self->{data};
    
    # if someone calls dump with different parameters, we have to invalidate
    # the cache (doesn't happen in normal pmx use).
    if ($self->{dump_params}) {
        delete $self->{dcache}
            if (exists $self->{dcache_kvsep} and $kvsep ne $self->{dcache_kvsep})
            or (exists $self->{dcache_psep} and $psep ne $self->{dcache_psep});
    }
        
    return $self->{dcache} if exists $self->{dcache};
    
    # find mods
    my %mods = ();
    my $i = 0;
    for (; $i <= $#$data; $i+=4) {
        if ($data->[$i+1] == $view)
        {
            my $lc_key = lc $data->[$i];
            $mods{$lc_key}->{$data->[$i+2]}->{key} = $data->[$i];
            $mods{$lc_key}->{$data->[$i+2]}->{val} = $data->[$i+3];
        }
        elsif (!$data->[$i+1]) {
            # stop when we hit the original data
            last;
        }
    }

    my $dump;
    my $last;
    for (;$i <= $#$data; $i+=4) {
        my $lc_key = lc $data->[$i];
        my $index = $data->[$i+2];
        
        # use the modified entry if it exists
        if (exists $mods{$lc_key}->{$index}) {
            $dump .= $mods{$lc_key}->{$index}->{key}.$kvsep.$mods{$lc_key}->{$index}->{val}.$psep
                if defined $mods{$lc_key}->{$index}->{val};
            delete $mods{$lc_key}->{$index};
            delete $mods{$lc_key} unless %{$mods{$lc_key}};
        }
        # else use the original data
        else {
            $dump .= $data->[$i].$kvsep.$data->[$i+3].$psep;
        }
        
        # try to put new entries beside existing ones of the same name, if there are any
        if ($last and $lc_key ne $last and exists $mods{$lc_key}) {
            my @keys = CORE::keys %{$mods{$lc_key}};
            foreach (sort @keys) {
                if (defined $mods{$lc_key}->{val}) {
                    $dump .= $mods{$lc_key}->{$_}->{key}.$kvsep.$mods{$lc_key}->{$_}->{val}.$psep;
                    delete $mods{$lc_key}->{$_};
                    delete $mods{$lc_key} unless CORE::keys %{$mods{$lc_key}};
                }
            }
        }

        $last = $lc_key;
    }
    
    # here we append all the new entries
    foreach (CORE::keys %mods) {
        my $lc_key = lc $_;
        my @keys = CORE::keys %{$mods{$lc_key}};
        foreach my $index (sort @keys) {
            $dump .= $mods{$lc_key}->{$index}->{key}.$kvsep.$mods{$lc_key}->{$index}->{val}.$psep
                if defined $mods{$lc_key}->{$index}->{val};
        }
    }
    
    # add the final separator
    $dump .= $psep;

    $self->{dcache} = $dump;
    $self->{dcache_kvsep} = $kvsep;
    $self->{dcache_psep} = $psep;

    return $dump;
}

sub get_replacements {
    my $self = shift;
    my $view = $self->{view};
    my $data = $self->{data};    
    
    my @mods = ();
    for (my $i = 0; $i <= $#$data; $i+=4) {
        if ($data->[$i+1] == $view)
        {
            push @mods, $data->[$i];
            push @mods, $data->[$i+2];
            push @mods, defined($data->[$i+3]) ? $data->[$i+3] : "";
        }
        elsif (!$data->[$i+1]) {
            # stop when we hit the original data
            last;
        }
    }
    return @mods;
}

sub get_additions {
    my $self = shift;
    my $view = $self->{view};
    my $data = $self->{data};    
    
    my @mods = ();
    # start from the end and go backwards
    for (my $i = scalar @$data - 4; $i >= 0; $i -= 4) {
        if ($data->[$i+1] == $view)
        {
            push @mods, $data->[$i];
            push @mods, defined($data->[$i+3]) ? $data->[$i+3] : "";
        }
        elsif (!$data->[$i+1]) {
            # stop when we hit the original data
            last;
        }
    }
    return @mods;
}


1;

=head1 NAME

PureMessage::Policy::VHash - Ordered key/value collection

=head1 SYNOPSIS

  # create object
  my $vhash = PureMessage::Policy::VHash->new;

  # update
  $vhash->put('key', 'value', $index);
  $vhash->append('key','value');

  # lookup
  my $value = $vhash->get('key', $index);
  my @keys = $vhash->items;
  print "key exists" if $vhash->exists($key);

  # dump it back as an RFC 822 header style string
  my $hdr = $vhash->dump;

=head1 DESCRIPTION

A C<PureMessage::Policy::VHash> object is a collection of key/value
pairs with the following features:

=over 2

=item *

Keys are strings.  Key case is preserved, but key lookup is not case
sensitive.

=item *

There can multiple entries with the same key.  When accessing entries
an index can be provided to designate which entry to address.

=item *

The order of key/value pairs is preserved.

=item *

Modifications to the collection are tracked and can be differentiated
from original data.

=back

The following methods are provided:

=over 4

=item $vhash = PureMessage::Policy::VHash->new

This is the constructor method that creates a new
C<PureMessage::Policy::VHash> object.  The object constructor does
not take any arguments.  The initial object will be empty and its view
set to 0.

=item $view = $vhash->view

=item $vhash->view( $new )

This method can be used to set/get the "view" into the collection.
Currently there are two supported views; 0 and 1.  View 0 corresponds
to the original data, and view 1 corresponds to modifications on the
data.

=item $value = $vhash->get( $key )

=item @values = $vhash->get( $key )

=item $value = $vhash->get( $key, $n )

Returns the value or values associated with $key.  Key lookup is case
insensitive.

In scalar context the value of the I<n>-th entry with the given key is
returned.  If there is no I<n>-th entry with the given key, then
C<undef> is returned.  The $n argument is optional and default to 0
which means return the first.

In list context all values with the given key are returned and $n
is ignored if provided.

=item @keys = $vhash->keys

Returns all the distinct keys in the collection.  In scalar context
the number of distinct keys are returned.

=item $bool = $vhash->exists( $key )

Returns TRUE iff an entry with the given $key exists.

=item $vhash->put( $key, $value )

=item $vhash->put( $key, $value, $n )

This method sets the value associated with I<n>-th entry with the
given $key.  The $n is assumed to be 0 if not specified.  If the
I<n>-th entry with the given key already exists, then its value is
overwritten by $value.  If not, a new entry is added.  If $value is
C<undef>, this is equivalent to deleting the entry.  Even though
put() is case-insensitive when matching an existing key, the case of
the key is set by the latest put() call on that key. 

=item $vhash->append( $key, $value )

Add a new key/value pair.  If there are already I<n> occurences of
$key, this new entry will have an index of I<n>.

=item $str = $vhash->dump

=item $str = $vhash->dump( $kvsep, $psep )

Dump the contents of the vhash.  Returns the entries of the vhash as a
formatted string in the following format:

    "key0${kvsep}value0${psep}key1${kvsep}value1${psep}...${psep}"

By default $kvsep is ": " and $psep is "\r\n" which would produce
something that looks like a RFC822 header.  Note that there will be
two occurences of $psep at the end of the string, unless the vhash is
empty in which case a single $psep is returned.

=item $vhash->get_replacements

Returns an array of "replacement" modifications (i.e. put() calls that
replaced original data).  The format of the array is:

    $key0, $index0, $value0, $key1, $index1, $value1, ...

=item $vhash->get_additions

Returns an array of "addition" modifications (i.e. put() calls that
did not replace original data, or append() calls).  The format of the
array is:

    $key0, $value0, $key1, $value1, ...

=back

=head1 SEE ALSO

L<pmx-policy>
